/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2004 by David Gerber <zapek@morphos.net>
 * Copyright 2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: lib.c,v 1.4 2006/08/08 13:31:43 fab Exp $
 */

#include "globals.h"

#if !USE_LEGACY
#include <libraries/query.h>
#endif

#include "lib.h"
#include "copyright.h"
#include "wb.h"

/*
 * Ah well, I'm tired of cut & past so define the stuff
 * and copy the file somewhere else.
 */
#define LIBNAME      "workbench.library"
#define LIBCOPYRIGHT " 2001-2004 David Gerber,  2006 Ambient Open Source Team"
#define LIBPRI 0
#define LIBBASE WorkbenchBase

char verstr[] = { "$VER: " LIBNAME " " LVERTAG " " LIBCOPYRIGHT };

extern ULONG LibFuncTable[];

struct Library *LIB_Init(struct LibBase	*LIBBASE, BPTR SegList, struct ExecBase *SBase);

struct libinitstruct
{
	ULONG	LibSize;
	void	*FuncTable;
	void	*DataTable;
	void	(*InitFunc)(void);
};

struct libinitstruct libinit_struct =
{
	sizeof(struct LibBase),
	LibFuncTable,
	NULL,
	(void (*)(void))&LIB_Init
};


struct Resident libresident = {
	RTC_MATCHWORD,
	&libresident,
	&libresident + 1,
	RTF_PPC | RTF_EXTENDED | RTF_AUTOINIT | RTF_AFTERDOS,
	VERSION,
	NT_LIBRARY,
	0,
	LIBNAME,
	LIBNAME " " LVERTAG " "LIBCOPYRIGHT,
	&libinit_struct,
	/* new fields */
	REVISION,
	(APTR)1,
};

/*
 * To tell the loader that this is a new emulppc elf and not
 * one for the ppc.library.
 */
ULONG __abox__ = 1;


/*
 * Library functions (system)
 */
struct Library *LIB_Init(struct LibBase *LIBBASE, BPTR SegList, struct ExecBase *SBase)
{
	LIBBASE->SegList = SegList;
	LIBBASE->SBase = SBase;

	if (lib_init(SBase))
	{
		LIBBASE->Lib.lib_Node.ln_Pri = LIBPRI;

		return (&LIBBASE->Lib);
	}

	lib_cleanup();

	FreeMem(((UBYTE *)LIBBASE) - LIBBASE->Lib.lib_NegSize,
		LIBBASE->Lib.lib_NegSize + LIBBASE->Lib.lib_PosSize);

	return (NULL);
}


/*
 * The following is needed because it's also called by LIB_Close() with
 * PPC args
 */
static BPTR libexpunge(struct LibBase *LIBBASE)
{
	BPTR MySegment;

	Forbid();
	if (LIBBASE->Lib.lib_OpenCnt)
	{
		LIBBASE->Lib.lib_Flags |= LIBF_DELEXP;
		Permit();
		return (NULL);
	}

	REMOVE((struct Node *)&LIBBASE->Lib);
	Permit();

	lib_cleanup();

	MySegment = LIBBASE->SegList;

	FreeMem(((UBYTE *)LIBBASE) - LIBBASE->Lib.lib_NegSize,
		LIBBASE->Lib.lib_NegSize + LIBBASE->Lib.lib_PosSize);

	return (MySegment);
}


static ULONG opened;


BPTR LIB_Expunge(void)
{
	struct LibBase *LIBBASE = (struct LibBase *)REG_A6;

	opened = FALSE;

	return (libexpunge(LIBBASE));
}



struct Library *LIB_Open(void)
{
	struct LibBase	*LIBBASE = (struct LibBase *)REG_A6;

	if (!opened)
	{
		opened = TRUE;

		if (!lib_open())
		{
			lib_cleanup();
			return (NULL);
		}
	}

	LIBBASE->Lib.lib_Flags &= ~LIBF_DELEXP;
	LIBBASE->Lib.lib_OpenCnt++;
	return (&LIBBASE->Lib);
}


BPTR LIB_Close(void)
{
	struct LibBase	*LIBBASE = (struct LibBase *)REG_A6;

	if ((--LIBBASE->Lib.lib_OpenCnt) == 0)
	{
		if (LIBBASE->Lib.lib_Flags & LIBF_DELEXP)
		{
			return (libexpunge(LIBBASE));
		}
	}
	return (0);
}


ULONG LIB_GetQueryAttr(void)
{
	#if !USE_LEGACY
	ULONG *data = (ULONG *)REG_A0;
	ULONG attr = REG_D0;

	if (data)
	{
		switch (attr)
		{
			case QUERYINFOATTR_NAME:
				*data = (ULONG)LIBNAME;
				return (TRUE);

			case QUERYINFOATTR_DESCRIPTION:
				*data = (ULONG)"Library to handle some desktop functions";
				return (TRUE);

			case QUERYINFOATTR_COPYRIGHT:
				*data = (ULONG)LIBCOPYRIGHT;
				return (TRUE);

			case QUERYINFOATTR_AUTHOR:
				*data = (ULONG)"David Gerber, Ambient Open Source Team";
				return (TRUE);

			case QUERYINFOATTR_SUBTYPE:
				*data = QUERYSUBTYPE_LIBRARY;
				return (TRUE);

			case QUERYINFOATTR_CLASS:
				*data = QUERYCLASS_NONE;
				return (TRUE);

			case QUERYINFOATTR_SUBCLASS:
				*data = QUERYSUBCLASS_NONE;
				return (TRUE);
		}
	}
	#endif
	return (FALSE);
}

